"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.divideConvertBackSmallestUnit = exports.multiplyByDecimals = exports.convertBack = exports.ensureDecimal = exports.priceFormatter = void 0;
const currency_js_1 = __importDefault(require("currency.js"));
const decimal_js_1 = __importDefault(require("decimal.js"));
/**
 * Formats a given price according to the specified currency and pattern and remove trailing zeros.
 *
 * @param {Object} params - The parameters for the price formatter.
 * @param {number} params.price - The price to be formatted.
 * @param {Object} params.currency - The currency properties.
 * @param {string} params.currency.symbol - The currency symbol.
 * @param {number} params.currency.decimals - The number of decimal places for the currency.
 * @param {boolean} [params.fromSmallestUnit=false] - Whether the price is in the smallest unit of the currency.
 * @param {boolean} [params.negative=false] - The pattern to format the price.
 * @returns {string} The formatted price string.
 */
const priceFormatter = ({ price, currency: currencyProp, fromSmallestUnit = false, negative = false, }) => {
    const { symbol, decimals } = currencyProp;
    const ensuredDecimal = (0, exports.ensureDecimal)(price);
    const decimalValue = new decimal_js_1.default(fromSmallestUnit
        ? divideConvertBack(ensuredDecimal, decimals)
        : (0, exports.convertBack)(ensuredDecimal, decimals));
    return (0, currency_js_1.default)(decimalValue.toNumber(), {
        symbol,
        pattern: `${negative ? '-' : ''}! #`,
        precision: decimalValue.isInteger() ? 0 : decimals,
    }).format();
};
exports.priceFormatter = priceFormatter;
const ensureDecimal = (value) => {
    return value instanceof decimal_js_1.default ? value : new decimal_js_1.default(value);
};
exports.ensureDecimal = ensureDecimal;
const convertBack = (value, decimals) => {
    const decimalPlaces = (0, exports.ensureDecimal)(decimals);
    if (decimalPlaces.isZero()) {
        return value.round().toNumber();
    }
    // If any input was Decimal, return Decimal, otherwise return number
    return value.toDecimalPlaces(decimalPlaces.toNumber()).toNumber();
};
exports.convertBack = convertBack;
const multiplyByDecimals = (value, decimals) => {
    return value.times(new decimal_js_1.default(10).pow(decimals));
};
exports.multiplyByDecimals = multiplyByDecimals;
/**
 * Divide a value by decimals and convert back to the original unit, but if `returnSmallestUnit` is true, return the
 * smallest unit instead or main unit.
 * @param value
 * @param decimals
 * @param returnSmallestUnit
 */
const divideConvertBackSmallestUnit = (value, decimals, returnSmallestUnit) => {
    return returnSmallestUnit
        ? (0, exports.convertBack)(value, 0)
        : divideConvertBack(value, decimals);
};
exports.divideConvertBackSmallestUnit = divideConvertBackSmallestUnit;
const divideByDecimals = (value, decimals) => {
    return value.dividedBy(new decimal_js_1.default(10).pow((0, exports.ensureDecimal)(decimals).toNumber()));
};
const divideConvertBack = (value, decimals) => {
    return (0, exports.convertBack)(divideByDecimals(value, decimals), decimals);
};
